using System;

using Syntriax.Engine.Core;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class ColorRGBConverter : EngineTypeYamlSerializerBase<ColorRGB>
{
    private static readonly int SUBSTRING_START_LENGTH = nameof(ColorRGB).Length + 1;

    public override ColorRGB Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        string value = parser.Consume<Scalar>().Value;
        string insideParenthesis = value[SUBSTRING_START_LENGTH..^1];
        string[] values = insideParenthesis.Split(", ");
        return new ColorRGB(byte.Parse(values[0]), byte.Parse(values[1]), byte.Parse(values[2]));
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        ColorRGB rgb = (ColorRGB)value!;
        emitter.Emit(new Scalar($"{nameof(ColorRGB)}({rgb.R}, {rgb.G}, {rgb.B})"));
    }
}
