using Syntriax.Engine.Core;

namespace Syntriax.Engine.Systems.Time;

public class TickerBehaviour : StopwatchBehaviour, ITicker
{
    public Event<ITicker> OnTick { get; } = new();

    public double Period { get; set; } = 1f;
    public int TickCounter { get; private set; } = 0;

    private double nextTick = 0f;

    public override void Start()
    {
        TickCounter = 0;
        base.Start();
        nextTick = Time + Period;
    }

    public override void Update()
    {
        base.Update();

        while (Time > nextTick)
        {
            nextTick += Period;
            TickCounter++;
            OnTick?.Invoke(this);
        }
    }

    protected override void OnFinalize()
    {
        base.OnFinalize();

        TickCounter = 0;
        nextTick = 0f;
    }
}
